@@ -4,25 +4,30 @@ import shutil
 import logging
 import numpy as np
 import pandas as pd
-from os.path import join, exists, split, dirname
+from os.path import join, exists, dirname, splitext
 from d3m.container import Dataset
 from d3m.utils import path_to_uri
-from d3m.metadata import base as metadata_base
 from d3m.container.utils import save_container
 from d3m.metadata.problem import PerformanceMetricBase, TaskKeywordBase
+from d3m.container.dataset import FILE_EXTENSIONS
 
 logger = logging.getLogger(__name__)
 DATASET_ID = 'internal_dataset'
 
 
 def is_d3m_format(dataset_path, suffix):
+    """Checks whether the given dataset is in D3M format, or if it needs conversion.
+    """
     if isinstance(dataset_path, str) and exists(join(dataset_path, 'dataset_%s' % suffix, 'datasetDoc.json')):
         return True
 
     return False
 
 
-def is_d3m_collection(dataset_path, collection_type):
+def is_d3m_collection(dataset_path, suffix, collection_type):
+    """Checks whether the given D3M dataset is a "collection".
+    """
+    dataset_path = join(dataset_path, 'dataset_%s' % suffix, 'datasetDoc.json')
     with open(dataset_path) as fin:
         dataset_doc = json.load(fin)
         for data_resource in dataset_doc['dataResources']:
@@ -33,13 +38,25 @@ def is_d3m_collection(dataset_path, collection_type):
 
 
 def dataset_to_d3m(dataset_path, output_folder, problem_config, suffix):
+    """Converts a "raw" dataset to D3M format and returns the temporary dataset directory.
+    """
     problem_config = check_problem_config(problem_config)
-    dataset_folder = join(output_folder, 'temp', 'dataset_d3mformat', suffix, 'dataset_%s' % suffix)
-    problem_folder = join(output_folder, 'temp', 'dataset_d3mformat', suffix, 'problem_%s' % suffix)
-    dataset = create_d3m_dataset(dataset_path, dataset_folder, problem_config)
-    create_d3m_problem(dataset['learningData'], problem_folder, problem_config)
+    d3m_root_folder = join(output_folder, 'temp', 'dataset_d3mformat', suffix)
+    d3m_dataset_folder = join(d3m_root_folder, 'dataset_%s' % suffix)
+    d3m_problem_folder = join(d3m_root_folder, 'problem_%s' % suffix)
+    d3m_dataset = create_d3m_dataset(dataset_path, d3m_dataset_folder)
+    create_d3m_problem(d3m_dataset['learningData'], d3m_problem_folder, problem_config)
 
-    return join(output_folder, 'temp', 'dataset_d3mformat', suffix)
+    if 'collection' in problem_config['extras']:
+        add_collection_resource(d3m_dataset['learningData'], d3m_dataset_folder, problem_config, dataset_path, suffix)
+
+    if 'time_indicator' in problem_config['extras']:
+        add_timeseries_info(d3m_dataset_folder, problem_config)
+
+    if 'objectDetection' in problem_config['task_keywords']:
+        modify_indexes(d3m_dataset_folder, problem_config)
+
+    return d3m_root_folder
 
 
 def check_problem_config(problem_config):
@@ -67,40 +84,37 @@ def check_problem_config(problem_config):
                          (problem_config['metric'], ', '.join(valid_metrics)))
 
     #  Check special cases
-    if problem_config['metric'] == 'f1' and 'binary' in problem_config['task_keywords'] and \
-            'pos_label' not in problem_config['optional']:
+    if 'f1' == problem_config['metric'] and 'binary' in problem_config['task_keywords'] and \
+            'pos_label' not in problem_config['extras']:
         raise ValueError('pos_label parameter is mandatory for f1 and binary problems')
 
-    if 'clustering' in problem_config['task_keywords'] and 'num_clusters' not in problem_config['optional']:
+    if 'clustering' in problem_config['task_keywords'] and 'num_clusters' not in problem_config['extras']:
         raise ValueError('num_clusters parameter is mandatory for clustering problems')
 
     if 'timeSeries' in problem_config['task_keywords'] and 'forecasting' in problem_config['task_keywords'] and \
-            'time_indicator' not in problem_config['optional']:
+            'time_indicator' not in problem_config['extras']:
         raise ValueError('time_indicator parameter is mandatory for time-series forecasting problems')
 
+    if 'timeSeries' in problem_config['task_keywords'] and 'grouped' in problem_config['task_keywords'] and \
+            'grouping_category' not in problem_config['extras']:
+        raise ValueError('grouping_category parameter is mandatory for grouped time-series forecasting problems')
+
+    if 'timeSeries' in problem_config['task_keywords'] and 'classification' in problem_config['task_keywords'] and \
+            'collection' not in problem_config['extras']:
+        raise ValueError('collection parameter is mandatory for time-series classification problems')
+
+    if any(x in problem_config['task_keywords'] for x in ['text', 'image', 'audio', 'video']) and \
+            'collection' not in problem_config['extras']:
+        raise ValueError('collection parameter is mandatory for text, image, audio, or video problems')
+
     return problem_config
 
 
-def create_d3m_dataset(dataset_path, destination_path, problem_config):
-    if callable(dataset_path):  # It's a sklearn dataset
-        dataset_path = 'sklearn://' + dataset_path.__name__.replace('load_', '')
+def create_d3m_dataset(dataset_path, destination_path):
     if exists(destination_path):
         shutil.rmtree(destination_path)
 
     dataset = Dataset.load(path_to_uri(dataset_path), dataset_id=DATASET_ID)
-
-    if 'time_indicator' in problem_config['optional']:
-        # Time indicator is needed by the primitive that splits time-series datasets in k-folds
-        column_metadata = {
-            'semantic_types': [
-                'https://metadata.datadrivendiscovery.org/types/Time', 'http://schema.org/DateTime',
-                'https://metadata.datadrivendiscovery.org/types/Attribute'
-            ]
-        }
-
-        time_index = dataset['learningData'].columns.get_loc(problem_config['optional']['time_indicator'])
-        dataset.metadata = dataset.metadata.update(('learningData', metadata_base.ALL_ELEMENTS, time_index), column_metadata)
-
     save_container(dataset, destination_path)
 
     return dataset
@@ -115,8 +129,8 @@ def create_d3m_problem(dataset, destination_path, problem_config):
     os.makedirs(destination_path)
 
     metric = {"metric": problem_config['metric']}
-    if 'pos_label' in problem_config['optional']:
-        metric['posLabel'] = str(problem_config['optional']['pos_label'])
+    if 'pos_label' in problem_config['extras']:
+        metric['posLabel'] = str(problem_config['extras']['pos_label'])
 
     target = {
         "targetIndex": 0,
@@ -124,8 +138,8 @@ def create_d3m_problem(dataset, destination_path, problem_config):
         "colIndex": problem_config['target_index'],
         "colName": problem_config['target_column']
     }
-    if 'num_clusters' in problem_config['optional']:
-        target["numClusters"] = problem_config['optional']['num_clusters']
+    if 'num_clusters' in problem_config['extras']:
+        target["numClusters"] = problem_config['extras']['num_clusters']
 
     problem_json = {
         "about": {
@@ -154,6 +168,65 @@ def create_d3m_problem(dataset, destination_path, problem_config):
         json.dump(problem_json, fout, indent=4)
 
 
+def add_collection_resource(dataset, d3m_dataset_folder, problem_config, dataset_path, suffix):
+    with open(join(d3m_dataset_folder, 'datasetDoc.json')) as fin:
+        dataset_json = json.load(fin)
+
+    collection_column = problem_config['extras']['collection']['column']
+    for column in dataset_json['dataResources'][0]['columns']:
+        if column['colName'] == collection_column:
+            column['role'].append('attribute')
+            column['colType'] = 'string'
+            column['refersTo'] = {'resID': '0', 'resObject': 'item'}
+
+    extension = splitext(dataset[collection_column].iat[0])[1]  # Only use the first element to get the extension
+    if extension not in FILE_EXTENSIONS:
+        raise ValueError('Unknown "%s" extension, you should choose: [%s]' % (extension, ', '.join(FILE_EXTENSIONS)))
+
+    collection_resource = {}
+    collection_resource['resID'] = '0'
+    collection_resource['resPath'] = 'collection'
+    collection_resource['resType'] = FILE_EXTENSIONS[extension].split('/')[0]
+    collection_resource['resFormat'] = {FILE_EXTENSIONS[extension]: [extension]}
+    collection_resource['isCollection'] = True
+    dataset_json['dataResources'].append(collection_resource)
+
+    mapping_folders = {'TRAIN': 'train_folder', 'TEST': 'test_folder', 'SCORE': 'test_folder'}
+    os.symlink(join(dirname(dataset_path), problem_config['extras']['collection'][mapping_folders[suffix]]),
+               join(d3m_dataset_folder, 'collection'))
+
+    with open(join(d3m_dataset_folder, 'datasetDoc.json'), 'w') as fout:
+        json.dump(dataset_json, fout, indent=4)
+
+
+def add_timeseries_info(d3m_dataset_folder, problem_config):
+    with open(join(d3m_dataset_folder, 'datasetDoc.json')) as fin:
+        dataset_json = json.load(fin)
+
+    time_column = problem_config['extras']['time_indicator']
+    grouping_column = problem_config['extras'].get('grouping_category', None)  # grouping_category can be optional
+
+    for column in dataset_json['dataResources'][0]['columns']:
+        if column['colName'] == time_column:
+            column['role'] += ['attribute', 'timeIndicator']
+            column['colType'] = 'dateTime'
+
+        if column['colName'] == grouping_column:
+            column['role'] += ['attribute', 'suggestedGroupingKey']
+            column['colType'] = 'categorical'
+
+    with open(join(d3m_dataset_folder, 'datasetDoc.json'), 'w') as fout:
+        json.dump(dataset_json, fout, indent=4)
+
+
+def modify_indexes(d3m_dataset_folder, problem_config):
+    csv_path = join(d3m_dataset_folder, 'tables', 'learningData.csv')
+    data = pd.read_csv(csv_path)
+    collection_column = problem_config['extras']['collection']['column']
+    data['d3mIndex'] = data.groupby(collection_column).ngroup()  # For the same values (e.g. images), the same indexes
+    data.to_csv(csv_path, index=False)
+
+
 def create_artificial_d3mtest(train_path, artificial_test_path, new_instances, target_column, text_column):
     # This is useful for cases where new instances are generated artificially
     if exists(artificial_test_path):
@@ -193,8 +266,9 @@ def create_artificial_d3mtest(train_path, artificial_test_path, new_instances, t
     data.to_csv(join(tables_folder, 'learningData.csv'), index=False)
 
 
-def d3mtext_to_dataframe(folder_path, text_column):
-    suffix = split(folder_path)[-1]
+def d3mtext_to_dataframe(folder_path, suffix, text_column):
+    """Reads a D3M text dataset into a single DataFrame, by loading the named text files.
+    """
     dataframe = pd.read_csv(join(folder_path, 'dataset_%s' % suffix, 'tables', 'learningData.csv'))
     folder_files = join(folder_path, 'dataset_%s' % suffix, 'media')
 
